import { Token, Tokens, TokenType, TokenManagerOptions, OktaAuth } from './types';
export declare const EVENT_EXPIRED = "expired";
export declare const EVENT_RENEWED = "renewed";
export declare const EVENT_ADDED = "added";
export declare const EVENT_REMOVED = "removed";
export declare const EVENT_ERROR = "error";
interface TokenError {
    errorSummary: string;
    errorCode: string;
    message: string;
    name: string;
    tokenKey: string;
}
declare type TokenErrorEventHandler = (error: TokenError) => void;
declare type TokenEventHandler = (key: string, token: Token, oldtoken?: Token) => void;
export declare class TokenManager {
    get: (key: string) => Promise<Token>;
    add: (key: string, token: Token) => void;
    clear: () => void;
    remove: (key: string) => void;
    renew: (key: string) => Promise<Token>;
    on: (event: string, handler: TokenErrorEventHandler | TokenEventHandler, context?: object) => void;
    off: (event: string, handler?: TokenErrorEventHandler | TokenEventHandler) => void;
    hasExpired: (token: Token) => boolean;
    getTokens: () => Promise<Tokens>;
    setTokens: (tokens: Tokens) => void;
    _getStorageKeyByType: (type: TokenType) => string;
    _clearExpireEventTimeoutAll: () => void;
    _getOptions: () => TokenManagerOptions;
    _resetExpireEventTimeoutAll: () => void;
    _emitEventsForCrossTabsStorageUpdate: (newValue: string, oldValue: string) => void;
    constructor(sdk: OktaAuth, options: TokenManagerOptions);
}
export {};
